package be.six.reports;

import java.io.File;
import java.util.Collections;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import be.six.reports.data.Query;
import be.six.reports.ui.MainFrame;
import be.six.reports.util.Configuration;
import be.six.reports.util.FileUtils;
import be.six.reports.util.Configuration.ConfigurationKey;

/**
 * Singleton used to start the application and to maintain state.
 * 
 * @author Joren Six
 * 
 */
public class Application {

	private static final Logger log = Logger.getLogger(Application.class
			.getName());
	private MainFrame mainFrame;

	/**
	 * Create the GUI and show it. For thread safety, this constructor uses the
	 * event-dispatching thread to initialize and show the JFrame.
	 */
	public Application() {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception e) {
					log.severe("Error setting native L&F: " + e);
				}
				
				checkFolders();
				
				mainFrame = new MainFrame();
			}
		});

	}

	/**
	 * Sets the application to 'wait' if there is a task in the background,
	 * removes the wait state otherwise. In wait state the application shows a
	 * wait cursor.
	 */
	public void toggleWait() {
		mainFrame.toggleWait();
	}

	private Vector<Query> queries;

	/**
	 * @return a list with the Query objects represented by the .sql files in
	 *         the queriesdirectory.
	 */
	public Vector<Query> queryList() {
		if (queries == null) {
			queries = new Vector<Query>();
			queries.addAll(Query.readQueriesFromDirectory());
			Collections.sort(queries);
		}
		return queries;
	}
	

	/**
	 * @param args
	 *            an empty list
	 */
	public static void main(String[] args) {
		instance = new Application();
	}

	private static Application instance;

	/**
	 * @return the only application instance
	 */
	public static Application getInstance() {
		return instance;
	}

	private void checkFolders() {
		boolean showMessage = false;
		String queryiesFolder = Configuration.get(ConfigurationKey.queries_folder);
		String exportFolder = Configuration.get(ConfigurationKey.export_folder);
		
		if(!folderExists(ConfigurationKey.queries_folder) && "queries/".equals(queryiesFolder)){
			 boolean success = (new File(queryiesFolder)).mkdir();
		    if (success) {
		    	String[] exampleQueries = {"data/example/ui-elements.sql","data/example/order-subtotals.sql","data/example/select-category.sql","data/example/orders-by-costumer.sql"};
		    	int i = 0;
		    	for (String example : exampleQueries){
		    		String contents = FileUtils.readFileFromJar(example);
		    		FileUtils.writeFile(contents, queryiesFolder + i + ".sql");
		    		i++;	
		    	}
		    }
		}
		
		while (!folderExists(ConfigurationKey.queries_folder)
				&& !queryFolderContainsSqlFiles(showMessage)) {
			selectFolder("Selecteer de map met queries (*.sql bestanden)",
					ConfigurationKey.queries_folder);
			showMessage = true;
		}
		
			
						
		if("exports/".equals(exportFolder) && !folderExists(ConfigurationKey.export_folder)){
			new File(exportFolder).mkdir();
		}
		while (!folderExists(ConfigurationKey.export_folder)) {
			selectFolder("Selecteer de (CSV) export map",
					ConfigurationKey.export_folder);
		}
		


	}

	public void selectFolder(final String title, final ConfigurationKey key) {
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory(new java.io.File("."));
		chooser.setDialogTitle(title);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			String dir = chooser.getSelectedFile().getAbsolutePath();
			if (!dir.endsWith(System.getProperty("file.separator")))
				dir = dir + System.getProperty("file.separator");
			Configuration.set(key, dir);
		}
	}

	private boolean folderExists(ConfigurationKey key) {
		File f = new File(Configuration.get(key));
		return f.exists() && f.isDirectory();
	}

	private boolean queryFolderContainsSqlFiles(boolean showMessage) {
		try {
			Query.readQueriesFromDirectory();
			return true;
		} catch (Error e) {
			if (showMessage)
				JOptionPane.showMessageDialog(null,
						"Geen sql files in de query directory");
			return false;
		}
	}
}
