package be.six.reports.data;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import be.six.reports.util.Configuration;
import be.six.reports.util.StopWatch;
import be.six.reports.util.Configuration.ConfigurationKey;

/**
 * The Database class represents a database. 
 * The connection parameters and driver are configured trough the java preferences API:
 *  http://java.sun.com/j2se/1.4.2/docs/guide/lang/preferences.html
 * The specified database driver has to be available in the ClassPath.
 * @author Joren Six
 */
public class Database {
	private static final Logger log = Logger.getLogger(Database.class.getName());
	private static Database instance;
	
	private Connection connection;
	
	/**
	 * @return The only Database instance.
	 */
	public synchronized static Database getInstance(){
		if(instance==null)
			instance = new Database();
		return instance;
	}
	
	/**
	 * Initializes the database driver and the connection. If the database driver is not found in the class path, an exception is flung.
	 */
	private Database(){
		String driver = Configuration.get(ConfigurationKey.database_driver);
		String connectionstring = Configuration.get(ConfigurationKey.database_connectionstring);
		String login = Configuration.get(ConfigurationKey.database_login);
		String password = Configuration.get(ConfigurationKey.database_password);
		try {
			Class.forName(driver);
			connection = DriverManager.getConnection(connectionstring,login,password);
		} catch (ClassNotFoundException e) {
			log.severe("Error initializing database driver connection");
			e.printStackTrace();
		}catch (SQLException e) {
			log.severe("Error initializing database connection");
			e.printStackTrace();
		}
	}
	
	
	/**
	 * 
	 * @param sql the select query to execute
	 * @return a DatabaseResult containing the 
	 * @throws SQLException
	 */
	public DatabaseResult fetchData(String sql) throws SQLException{
		StopWatch watch = new StopWatch();
		watch.start();
		if(connection==null)
			return DatabaseResult.EMPTYRESULT;
		
		Statement stmt = connection.createStatement();
		
		ResultSet rs = stmt.executeQuery(sql);
		DatabaseResult result = new DatabaseResult(rs.getMetaData());
		while (rs.next()) {
			result.addRow(rs);
		}
		rs.close();
		stmt.close();
		watch.stop();
		log.info("SQL Executed in "+ watch.getElapsedString() +": \n\n" + sql + "\n");
		
		return result;
	}
	
	/**
	 * Sets the parameters in parameters in order and fetches data using a prepared statement.
	 * @param sql the sql with question mark parameters e.g. select * from table where id = ? and name like ?
	 * @param parameters a list of parameters e.g. [10,"%a name"] 
	 * @return A DatabaseResult representing the result.
	 * @throws SQLException when something goes awry.
	 */
	public DatabaseResult fetchData(String sql,List<Object> parameters) throws SQLException{
		StopWatch watch = new StopWatch();
		watch.start();
		PreparedStatement stmt = connection.prepareStatement(sql);
		int index=1;
		
		StringBuilder logText=new StringBuilder();
				
		for(Object param : parameters){
			if(param instanceof String || param instanceof Character)
				stmt.setString(index,param.toString());
			else if(param instanceof Boolean)
				stmt.setBoolean(index,((Boolean)param).booleanValue());
			else if(param instanceof Integer)
				stmt.setInt(index,(Integer)param);
			else if(param instanceof Date)
				stmt.setDate(index,(Date)param);
			else if(param instanceof java.util.Date)
				stmt.setDate(index,new Date(((java.util.Date)param).getTime()));
			else if(param instanceof Double)
				stmt.setDouble(index,(Double) param);
			else if(param instanceof Float)
				stmt.setDouble(index,(Float) param);
			logText.append(index).append(" => ").append(param).append("\n");
			index++;
		}
		
		log.info("Params:\n" + logText.toString());
		
		ResultSet rs = stmt.executeQuery();
		DatabaseResult result = new DatabaseResult(rs.getMetaData());
		while (rs.next()) {
			result.addRow(rs);
		}
		rs.close();
		stmt.close();
		watch.stop();
		log.info("SQL Executed in "+ watch.getElapsedString() +": \n\n" + sql + "\n");
		
		return result;
	}
	
	
	
	/**
	 * closes the database connection
	 */
	public void close(){
		try {
			this.connection.close();
			log.warning("Connection closed!");
		} catch (SQLException e) {
			log.warning("Connection could not be closed!");
			//ignore
		}
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#finalize()
	 */
	protected void finalize() throws Throwable{
	  close();
	}

	

	/**
	 * Queries the database with only one parameter.
	 * @param sql the SQL statement with one question mark parameter e.g. select * from table where id = ?
	 * @param param a parameter e.g. 10 
	 * @return A DatabaseResult representing the result.
	 */
	public DatabaseResult queryWithOneParam(String sql, Object param){
		List<Object> params = new ArrayList<Object>();
		params.add(param);
		try {
			return fetchData(sql,params);
		} catch (SQLException e) {
			log.severe("Error while fetching with one param: " + sql);
			e.printStackTrace();
		}
		return null;
	}
}
