package be.six.reports.data;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.logging.Logger;

/**
  * An offline representation of the result of a SELECT statement. It tries to convert database types to the equivalent Java type.
 * Watch out for results with more than 100k rows: the heap space needed will grow significantly. This could lead to heap space errors.
 * @author Joren Six
 */
public class DatabaseResult {
	private static final Logger log = Logger.getLogger(DatabaseResult.class.getName());
	public static final DatabaseResult EMPTYRESULT = new DatabaseResult();
	private final Class<?>[] columnClasses;
	private final int columnCount;
	private final String[] columnNames;
	private final ArrayList<Object[]> rowList ;
	
	//EmptyResult
	private DatabaseResult(){
		columnCount=0;
		columnClasses= new Class[columnCount];
		columnNames = new String[columnCount];
		rowList = new ArrayList<Object[]>();
	}
	
	/**
	 * Initializes the database result.
	 * @param metaData the meta data for this result. Contains information about the columns.
	 * @throws SQLException when using the meta data goes awry.
	 */
	public DatabaseResult(ResultSetMetaData metaData) throws SQLException{
		columnCount = metaData.getColumnCount();	
		columnClasses= new Class[columnCount];
		columnNames = new String[columnCount];
		rowList = new ArrayList<Object[]>();
		for(int i=0;i<columnCount;i++){
			int dbType = metaData.getColumnType(i+1);
			columnNames[i]=metaData.getColumnName(i+1);
			if("maandblad".equalsIgnoreCase(columnNames[i]))
				columnClasses[i] = Boolean.class;
			else
				columnClasses[i]=dbTypeToClass(dbType);
		}
	}
	
	/**
	 * Casts a database type to a java class.
	 * @param dbType the database type to cast.
	 * @return the java type corresponding with the database type. If no match is found String is assumed.
	 */
	private Class<?> dbTypeToClass(int dbType){
		switch (dbType) {
			case Types.BOOLEAN:
				return Boolean.class;
			case Types.INTEGER:
				return Integer.class;
			case Types.FLOAT:
				return Float.class;
			case Types.DOUBLE:
			case Types.REAL:
			case Types.NUMERIC:	
				return Double.class;
			case Types.DATE:
			case Types.TIME:
			case Types.TIMESTAMP:
				return java.sql.Date.class;
			case Types.CHAR:
			case Types.VARCHAR:
			case Types.LONGVARCHAR:
			case Types.OTHER:	
				return String.class;
			default:
				log.info(dbType + " => string" );
				return String.class;
		}
	}
	
	

	/**
	 * Adds a row to the DatabaseResult
	 * @param rs the ResultSet from a SELECT statement. 
	 * @throws SQLException when reading data goes awry.
	 */
	public void addRow(ResultSet rs) throws SQLException {
		ArrayList<Object> cellList = new ArrayList<Object>();
		for (int i = 0; i < columnCount; i++) {
			Object cellValue = null;
			if (columnClasses[i] == String.class)
				cellValue = rs.getString(columnNames[i]);
			else if (columnClasses[i] == Integer.class)
				cellValue = new Integer(rs.getInt(columnNames[i]));
			else if (columnClasses[i] == Float.class)
				cellValue = new Float(rs.getInt(columnNames[i]));
			else if (columnClasses[i] == Double.class)
				cellValue = new Double(rs.getDouble(columnNames[i]));
			else if (columnClasses[i] == java.sql.Date.class)
				cellValue = rs.getDate(columnNames[i]);
			else if (columnClasses[i] == Boolean.class)
				cellValue = rs.getInt(columnNames[i]) != 0;
			else
				log.severe("Can't assign " + columnNames[i]);
			cellList.add(cellValue);
		}
		
		Object[] cells = cellList.toArray();
		rowList.add(cells);		
	}

	/**
	 * @return the number of rows in the DatabaseResult
	 */
	public int getRowCount() {
		return rowList.size();
	}

	/**
	 * @return the number of columns in the databaseResult
	 */
	public int getColumnCount() {
		return columnCount;
	}

	/**
	 * @param col the column
	 * @return the class of the selected column
	 */
	public Class<?> getColumnClass(int col) {
		return columnClasses[col];
	}

	/**
	 * @param col the column
	 * @return the name of the column
	 */
	public String getColumName(int col) {
		return columnNames[col];
	}

	/**
	 * @param row the row
	 * @param column the column
	 * @return the value at the selected position
	 */
	public Object getValueAt(int row, int column) {
		return rowList.get(row)[column];
	}
	
}
