package be.six.reports.data;


import java.awt.Container;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;

import be.six.reports.Application;
import be.six.reports.util.Configuration;
import be.six.reports.util.FileUtils;
import be.six.reports.util.Configuration.ConfigurationKey;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * Represents a database query. Is responsible for executing the query it represents and for its own UI
 * @author Joren Six
 */
public class Query  implements Comparable<Query> {
	private static final Logger log = Logger.getLogger(Query.class.getName());
	
		
	/**
	 * The title of the query
	 */
	private final String title;
	/**
	 * A description
	 */
	private final String description;
	/**
	 * The SQL instruction text with parameters and all. E.g. "Select * FROM blaat WHERE bla = !{tekst param}!"
	 */
	private final String sql;
	/**
	 * list of parameters in SQL-text
	 */
	private final HashMap<String,Parameter> parametersByName; 
	private JComboBox queriesComboBox;
	private Container ui;
	
	private Query(String title,String description,String sql){
		this.title=title;
		this.description=description;
		this.sql=sql;
		parametersByName = new HashMap<String,Parameter>();
		initializeParameters();
	}
	
	/**
	 * Searches parameters in SQL-query text and puts them in a list of strings.
	 */
	private void initializeParameters(){
        Pattern p = Pattern.compile(Parameter.PATTERN, Pattern.MULTILINE);
        Matcher m = p.matcher(sql);
        while (m.find()){
        	String name = m.group(1);
        	if(!parametersByName.containsKey(name))
        		try{
        			parametersByName.put(name,Parameter.getParameter(name));
        		}catch(Error e){
        			log.severe(e.getMessage());
        		}
        }
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return title;
	}
	
	
	/**
	 * The user interface for this query.
	 * @return a Container with the user interface for this Query.
	 */
	public Container ui(){
		
		if (ui==null){
			FormLayout layout = new FormLayout("right:pref, 3dlu, min:grow");
	        DefaultFormBuilder builder = new DefaultFormBuilder(layout);
	        builder.setDefaultDialogBorder();
	        builder.setRowGroupingEnabled(true);
	        
	        queriesComboBox = new JComboBox(Application.getInstance().queryList());
	        
	        builder.appendSeparator("Kies de query");        
	        builder.append("Query:",queriesComboBox,true);
	       
	        JEditorPane desc = new JEditorPane();
	        desc.setContentType( "text/html" );
	        desc.setText("<HTML>" + description + "</HTML>");
			desc.setEditable(false);

			CellConstraints cc = new CellConstraints();
	     
	        builder.append("Omschrijving:");
	        builder.appendRow("31dlu"); // Assumes line is 14, gap is 3
	        builder.add(new JScrollPane(desc),cc.xywh(builder.getColumn(), builder.getRow(), 1, 2));
	        builder.nextLine(2);      
	        
	        if(this.parametersByName.size()!=0)
	        	builder.appendSeparator("Query parameters");
	      	        
	        addParametersToLayout(builder);	        
			
	        ui = builder.getPanel();
		}
		return ui;
	}
	
	
	/**
	 * Adds the parameter UI-elements to the layout and stores them in the hash
	 * @param builder the builder used to build the form
	 */
	private void addParametersToLayout( DefaultFormBuilder builder){
	    for(Parameter param: parametersByName.values()){
        	JComponent component = param.getComponent();
        	String name = param.getLabel();
        	builder.append(name,component,true);
        }
	}
	

	
	/**
	 * Fetches the values of the UI-elements in the hash
	 * @return an ordered list of parameter values.
	 */
	private List<Object> paramValues(){
		List<Object> objects = new ArrayList<Object>();
	    Pattern p = Pattern.compile(Parameter.PATTERN, Pattern.MULTILINE);
        Matcher m = p.matcher(sql);
        while (m.find()){
        	String name = m.group(1);
        	Parameter param = parametersByName.get(name);
        	if(!( param instanceof SQLParameter ))
        		objects.add(param.getValue());
        }
		return objects;
	}
	
	/**
	 * Executes this query.
	 * @return executes the query and returns a database result.
	 */
	public DatabaseResult execute(){
		
		try {
			return Database.getInstance().fetchData(sqlToExecute(),paramValues());
		} catch (SQLException e) {
			log.severe("Error tijdens het uitvoeren van \n\n ("+e.getMessage()+")" + sqlToExecute());
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	/**
	 * 
	 * @return replaces !{sql name}! fields with the text in the text fields.
	 */
	private  String sqlToExecute(){
		String sqlToExecute = sql;		
		//first replace raw sql:
		for(Parameter param : parametersByName.values()){
			if(param instanceof SQLParameter)
				sqlToExecute=((SQLParameter)param).replaceSQL(sqlToExecute);
		}
		//daarna overblijvende patronen vervangen door ?
		return sqlToExecute.replaceAll(Parameter.PATTERN,"?");
	}

	/**
	 * Parses the contents of a *.sql file.
	 * @param queryText the contents of a sql file
	 * @return a Query object respresenting this file. If the query can not be parsed null.
	 */
	private static Query parse(String queryText){
		String[] array = queryText.split("----");
		if(array.length==3){
			String title = array[0].trim();
			String description = array[1].trim();
			String sql = array[2].trim();
			return new Query(title,description,sql);			
		}else{
			log.severe("Invalid Query: " + queryText);
			return null;
		}
	}
	
	/**
	 * Returns a list of Query objects representing SQL-files in a directory.
	 * @return a list of Query objects representing every SQL-file in the configured directory.
	 */
	public static List<Query> readQueriesFromDirectory(){
		List<Query> queries = new ArrayList<Query>();
		String directory = Configuration.get(ConfigurationKey.queries_folder);
		String[] sqlFiles = sqlFilesInDirectory(directory);
		if(sqlFiles==null || sqlFiles.length==0){
			throw new Error("Configure the query directory!");
		}
		for(String fileName : sqlFiles){
			String contents = FileUtils.readFile(directory + fileName);
			Query q = parse(contents);
			if(q!=null)queries.add(q);
		}
		return queries;
	}
	
	/**
	 * 
	 * @param directory the directory to search
	 * @return null if the directory is not accessible. An empty array if no SQL files are found, otherwise an array with filenames of the SQL files in the directory is returned.  
	 */
	private static String[] sqlFilesInDirectory(String directory){
		FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return !name.startsWith(".") && name.toLowerCase().endsWith(".sql");
	        }
	    };
	    File dir = new File(directory);
	    return dir.list(filter);
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Query o) {
		return this.title.compareTo(o.title);
	}
	


	/**
	 * Executes this query and exports the result to the configured directory
	 * @return the DatabaseResult
	 */
	public DatabaseResult export() {
		DatabaseResult result = this.execute();
		FileUtils.Export(title, result);
		return result;
	}

	/**
	 * Adds an action listener to this.
	 * @param actionListener the action listener 
	 */
	public void addActionListener(ActionListener actionListener) {
		queriesComboBox.addActionListener(actionListener);
	}	
	
	/**
	 * Sets the selected query.
	 * @param item the query to select
	 */
	public void setSelectedItem(Object item){
		this.queriesComboBox.setSelectedItem(item);		
	}
}
