package be.six.reports.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Logger;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

/**
 * Utility class to access configuration parameters
 * @author Joren Six
 * 
 */
public class Configuration {
	private static final Logger log = Logger.getLogger(Configuration.class.getName());

	private static Properties defaultConfigurationProperties;
	private static Preferences userPreferences = null;
	
	/**
	 * Defines a configuration key.
	 * @author Joren Six
	 * 
	 */
	public enum ConfigurationKey{
		/**
		 * The folder with SQL queries, e.g. /home/user/queries/ or C:/queries/ the last / is mandatory
		 */
		queries_folder,
		/**
		 * The folder where the CSV exports are saved, , e.g. /home/user/export/ or C:/export/ the last / is mandatory
		 */
		export_folder,
		/**
		 * The JDBC database driver, e.g.  oracle.jdbc.OracleDriver
		 */
		database_driver,
		/**
		 * The JDBC connection string, e.g. jdbc:oracle:thin:@163.194.21.35:1521:INSTANCE
		 */
		database_connectionstring,
		/**
		 * The login for the database user 
		 */
		database_login,
		/**
		 * The password for the database user
		 */
		database_password,
		/**
		 * Queries to populate comboboxes with. Defined by:
		 * <br> 
		 * <code>name: SQL Query [|  othername : other SQL Query]</code>
		 * <br>
		 * The sql query should yield a result with two columns, an id and a description. E.g.:
		 * <br>
		 * <code>users: select id,name from users  | categories: select id,desc from categories</code>  
		 */
		list_queries,
		/**
		 * The SimpleDateFormat string used in the csv-export. E.g <code>yyyy-MM-dd hh:mm:ss</code>
		 */
		date_format,
		/**
		 * The NumberFormat string used in the csv-export. E.g. <code>0.##########</code>
		 */
		number_format,
		/**
		 * The CSV field separator
		 */
		CSV_separator,
		/**
		 * The SimpleDateFormat string used in the filename of the csv-export. E.g <code>yyyy-MM-dd hh-mm-ss</code>
		 */
		CSV_filename_timstamp_format
	}
	

	/**
	 * Read the configuration for a certain key. 
	 * If the key can not be found in the UserPreferences the default is loaded from
	 *  a properties file, written in the UserPreferences, and returned. Otherwise the 
	 *  configured value is returned.  
	 * @param key the configuration key
	 * @return if the key is configured, the configured value. Otherwise the default value is returned.
	 */
	public static String get(ConfigurationKey key){		
		return get(key.name());
	}
	
	public static void set(ConfigurationKey key,String value){
		try {
			userPreferences.put(key.name(), value);
			userPreferences.flush();
			log.info(key.name() + " = " + value );
		} catch (BackingStoreException e) {
			log.severe("Could not save preference for " + key.name());
			e.printStackTrace();
		}
	}

	private static String get(String key) {
		if(defaultConfigurationProperties==null){			
			defaultConfigurationProperties = new Properties();
			InputStream propertiesStream = Configuration.class.getResourceAsStream("configuration.properties");
			try {
				defaultConfigurationProperties.load(propertiesStream);
			} catch (IOException e) {
				log.severe("Could not find default database configuration properties file");
				throw new Error(e);
			}
		}
		String defaultValue=defaultConfigurationProperties.getProperty(key);
		if(userPreferences==null){
			userPreferences=Preferences.userRoot();
			for(ConfigurationKey configKey : ConfigurationKey.values()){
				String defaultConfigValue=defaultConfigurationProperties.getProperty(configKey.name());
				//als de gebruiker nog niets heeft geconfigureerd wordt de default configuratie eenmalig weggeschreven
				if(userPreferences.get(configKey.name(), defaultConfigValue).equals(defaultConfigValue)){
					userPreferences.put(configKey.name(),defaultConfigValue);
				}
				try {
					userPreferences.flush();
				} catch (BackingStoreException e) {
					log.severe("Could not write log values: " + e.getMessage()) ;
					e.printStackTrace();
				}
			}
		}					
		return userPreferences.get(key, defaultValue);
		
	}
}
