package be.six.reports.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.logging.Logger;

import be.six.reports.Application;
import be.six.reports.data.DatabaseResult;
import be.six.reports.util.Configuration.ConfigurationKey;

/**
 * Exports a DatabaseResult to a CSV-file.
 * @author Joren Six
 * 
 */
public class FileUtils {
	private static final Logger log = Logger.getLogger(FileUtils.class.getName());
	
	//disable default constructor
	private FileUtils(){}
	
	/**
	 * Write the CSV-file
	 * @param filename The name of the file.
	 * @param resultToExport The DatabaseResult to export.
	 */
	public static void Export(String filename,DatabaseResult resultToExport){
		String directory = Configuration.get(ConfigurationKey.export_folder);
		
		String dateFormat = Configuration.get(ConfigurationKey.date_format);
		String numberFormat = Configuration.get(ConfigurationKey.number_format);
		String filenameTimstampFormat = Configuration.get(ConfigurationKey.CSV_filename_timstamp_format);
		
		SimpleDateFormat filenameTimestampFormatter = new SimpleDateFormat (filenameTimstampFormat);
		SimpleDateFormat exportDateFormatter = new SimpleDateFormat (dateFormat);
		DecimalFormat exportDecimalFormat = new DecimalFormat(numberFormat);
		
		String separator = Configuration.get(ConfigurationKey.CSV_separator);
		
		Date currentTime = new Date();
		String dateString = filenameTimestampFormatter.format(currentTime); 
		
		filename = directory + dateString + filename + ".csv";
		FileWriter FW=null;
		try {
			FW = new FileWriter(filename);
			BufferedWriter OutputStream=new BufferedWriter(FW);
			PrintWriter output =new PrintWriter(OutputStream);
			
			//HEADERS
			for (int column = 0; column < resultToExport.getColumnCount(); column++) {
				Object valueObject = resultToExport.getColumName(column);
				String value =valueObject==null?"":valueObject.toString();
				value = value.replace(separator,"");
				output.print(value + separator );
			}
			output.println("");
			
			//DATA
			for (int row = 0; row < resultToExport.getRowCount(); row++) {
				for (int column = 0; column < resultToExport.getColumnCount(); column++) {
					Object valueObject = resultToExport.getValueAt(row, column);
					String value =valueObject==null?"":valueObject.toString();
					if(valueObject != null){
						if(valueObject instanceof Double) value = exportDecimalFormat.format(valueObject);
						else if(valueObject instanceof Date ) value = exportDateFormatter.format(valueObject);
					}
					value = value.replace(separator,"");
					output.print(value + separator);
				}
				output.println("");
			}
			OutputStream.flush(); 
			OutputStream.close();
		} catch (IOException i1) {
			log.severe("Can't open file:" + filename );
		}
	}
	
	
	/**
	 * Writes a file to disk. Uses the string contents as content. Fails silently.
	 * @param contents The contents of the file.
	 * @param name The name of the file to create.
	 * 
	 */
	public static void writeFile(String contents, String name){
		FileWriter FW=null;
		try {
			FW = new FileWriter(name);
			BufferedWriter outputStream=new BufferedWriter(FW);
			PrintWriter output = new PrintWriter(outputStream);
			output.print(contents);
			outputStream.flush(); 
			outputStream.close();
		} catch (IOException i1) {
			log.severe("Can't open file:" + name );
		}
	}
	
	/**
	 * Reads the contents of a file.
	 * @param name the name of the file to read
	 * @return the contents of the file if successful, an empty string otherwise.
	 */
	public static String readFile(String name){
		FileReader fileReader=null;
		StringBuilder contents = new StringBuilder();
		try {
			File file = new File(name);
			if(!file.exists())
				throw new Error("File " + name + " does not exist");
			fileReader=new FileReader(file);
			BufferedReader in = new BufferedReader(fileReader);
			String inputLine;
			while ((inputLine = in.readLine()) != null) 
				contents.append(inputLine).append("\n");
			in.close();
		} catch (IOException i1) {
			log.severe("Can't open file:" + name );
		}
		return contents.toString();
	}
	
	/**
	 * Reads the contents of a file in the jar.
	 * @param path the path to read e.g. /be/six/reports/ui/help.html
	 * @return the contents of the file when successfull, an empty string otherwise.
	 */
	public static String readFileFromJar(String path){
		StringBuilder contents = new StringBuilder();
		URL url = Application.class.getResource(path);
		URLConnection connection;
		try {
			connection = url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
			String inputLine;
			while ((inputLine = in.readLine()) != null) 
				contents.append(new String(inputLine.getBytes(),"UTF-8")).append("\n");
			in.close();
		} catch (IOException e) {
			log.severe("Error while reading file " + path + " from jar: " + e.getMessage());
			e.printStackTrace();
		}  catch (NullPointerException e) {
			log.severe("Error while reading file " + path + " from jar: " + e.getMessage());
			e.printStackTrace();
		} 	
		return contents.toString();	
	}
	
}
